@@ -562,7 +562,7 @@ void _dl_boot(int args){
 	c = *str2;
 	*str2 = '\0';
 	if (!_dl_secure || _dl_strchr(str, '/') == NULL) {
-	  tpnt1 = _dl_load_shared_library(NULL, str);
+	  tpnt1 = _dl_load_shared_library(_dl_secure, NULL, str);
 	  if (!tpnt1) {
 	    if (_dl_trace_loaded_objects)
 	      _dl_fdprintf(1, "\t%s => not found\n", str);
@@ -640,7 +640,7 @@ void _dl_boot(int args){
 	      c = *cp;
 	      *cp = '\0';
 
-	      tpnt1 = _dl_load_shared_library(NULL, cp2);
+	      tpnt1 = _dl_load_shared_library(0, NULL, cp2);
 	      if (!tpnt1) {
 		if (_dl_trace_loaded_objects)
 		  _dl_fdprintf(1, "\t%s => not found\n", cp2);
@@ -703,7 +703,7 @@ void _dl_boot(int args){
 	    tpnt = NULL;
 	    continue;
 	  }
-	  if (!(tpnt1 = _dl_load_shared_library(tcurr, lpnt)))
+	  if (!(tpnt1 = _dl_load_shared_library(0, tcurr, lpnt)))
 	  {
 	    if (_dl_trace_loaded_objects)
 	      _dl_fdprintf(1, "\t%s => not found\n", lpnt);
