@@ -1,4 +1,4 @@
-#include <link.h>
+#include "link.h"
 
 #ifndef RTLD_NEXT
 #define RTLD_NEXT	((void*)-1)
