@@ -1,11 +1,12 @@
 /*
  * ldconfig - update shared library symlinks
  *
- * usage: ldconfig [-DvnNX] [-f conf] [-C cache] [-r root] dir ...
+ * usage: ldconfig [-DvqnNX] [-f conf] [-C cache] [-r root] dir ...
  *        ldconfig -l [-Dv] lib ...
  *        ldconfig -p
  *        -D: debug mode, don't update links
  *        -v: verbose mode, print things as we go
+ *        -q: quiet mode, don't print warnings
  *        -n: don't process standard directories
  *        -N: don't update the library cache
  *        -X: don't update the library links
@@ -39,6 +40,12 @@
 #include <errno.h>
 
 #include "../config.h"
+#include "readelf.h"
+
+#ifdef __GNUC__
+void warn(const char *fmt, ...) __attribute__ ((format (printf, 1, 2)));
+void error(const char *fmt, ...) __attribute__ ((format (printf, 1, 2)));
+#endif
 
 char *___strtok = NULL;
 
@@ -63,18 +70,20 @@ int libmode = 0;		/* library mode */
 int nocache = 0;		/* don't build cache */
 int nolinks = 0;		/* don't update links */
 
-char *conffile = LDSO_CONF;	/* default cache file */
+char *conffile = LDSO_CONF;	/* default conf file */
 char *cachefile = LDSO_CACHE;	/* default cache file */
 void cache_print(void);
-void cache_dolib(char *dir, char *so, int libtype);
+void cache_dolib(const char *dir, const char *so, int libtype);
 void cache_write(void);
 
-char *readsoname(char *name, FILE *file, int *type);
-
-void warn(char *fmt, ...)
+void warn(const char *fmt, ...)
 {
     va_list ap;
 
+    if (verbose < 0)
+        return;
+
+    fflush(stdout);    /* don't mix output and error messages */
     fprintf(stderr, "%s: warning: ", prog);
 
     va_start(ap, fmt);
@@ -86,10 +95,11 @@ void warn(char *fmt, ...)
     return;
 }
 
-void error(char *fmt, ...)
+void error(const char *fmt, ...)
 {
     va_list ap;
 
+    fflush(stdout);    /* don't mix output and error messages */
     fprintf(stderr, "%s: ", prog);
 
     va_start(ap, fmt);
@@ -109,18 +119,31 @@ void *xmalloc(size_t size)
     return ptr;
 }
 
-char *xstrdup(char *str)
+char *xstrdup(const char *str)
 {
     char *ptr;
-    char *strdup(const char *);
     if ((ptr = strdup(str)) == NULL)
 	error("out of memory");
     return ptr;
 }
 
-/* if shared library, return a malloced copy of the soname and set the
-   type, else return NULL */
-char *is_shlib(char *dir, char *name, int *type, int *islink)
+/* If shared library, return a malloced copy of the soname and set the
+   type, else return NULL.
+
+   expected_type should be either LIB_ANY or one of the following:-
+   LIB_DLL
+   LIB_ELF
+   LIB_ELF_LIBC5
+   LIB_ELF_LIBC6
+
+   If the lib is ELF and we can not deduce the type the type will
+   be set based on expected_type.
+
+   If the expected, actual/deduced types missmatch we display a warning
+   and use the actual/deduced type.
+*/
+char *is_shlib(const char *dir, const char *name, int *type,
+	int *islink, int expected_type)
 {
     char *good = NULL;
     char *cp, *cp2;
@@ -148,8 +171,7 @@ char *is_shlib(char *dir, char *name, int *type, int *islink)
 	if (lstat(buff, &statbuf))
 	    warn("can't lstat %s (%s), skipping", buff, strerror(errno));
 	else if (!S_ISREG(statbuf.st_mode) && !S_ISLNK(statbuf.st_mode))
-	    warn("%s is not a regular file or symlink, skipping", 
-		 buff, strerror(errno));
+	    warn("%s is not a regular file or symlink, skipping", buff);
 	else
 	{
 	    /* is it a regular file or a symlink */
@@ -176,8 +198,12 @@ char *is_shlib(char *dir, char *name, int *type, int *islink)
 		    else
 		    {
 		        /* always call readsoname to update type */
+			if(expected_type == LIB_DLL){
+			    warn("%s is not an a.out library, its ELF!\n", buff);
+			    expected_type=LIB_ANY;
+			}
 		        *type = LIB_ELF;
-		        good = readsoname(buff, file, type);
+		        good = readsoname(buff, file, expected_type, type);
 			if (good == NULL || *islink)
 			{
 			    if (good != NULL)
@@ -206,6 +232,12 @@ char *is_shlib(char *dir, char *name, int *type, int *islink)
 			strncpy(good, name, cp - name);
 			good[cp - name] = '\0';
 		    }
+		    if(expected_type != LIB_ANY && expected_type != LIB_DLL)
+		    {
+			warn("%s is not an ELF library, its an a.out DLL!", buff);
+			expected_type=LIB_ANY;
+		    }
+		    
 		    *type = LIB_DLL;
 		}
 		fclose(file);
@@ -217,7 +249,7 @@ char *is_shlib(char *dir, char *name, int *type, int *islink)
 }
 
 /* update the symlink to new library */
-void link_shlib(char *dir, char *file, char *so)
+void link_shlib(const char *dir, const char *file, const char *so)
 {
     int change = -1;
     char libname[1024];
@@ -252,7 +284,7 @@ void link_shlib(char *dir, char *file, char *so)
     }
 
     /* some people like to know what we're doing */
-    if (verbose)
+    if (verbose > 0)
 	printf("\t%s => %s%s\n", so, file,
 	       change < 0 ? " (SKIPPED)" :
 	       (change > 0 ? " (changed)" : ""));
@@ -297,16 +329,48 @@ struct lib
 };
 
 /* update all shared library links in a directory */
-void scan_dir(char *name)
+void scan_dir(const char *name)
 {
     DIR *dir;
     struct dirent *ent;
     char *so;
     struct lib *lp, *libs = NULL;
     int libtype, islink;
+    int expected_type = LIB_ANY;
+    char *t;
+
+    /* Check for an embedded expected type */
+    t=strrchr(name, '=');
+    if( t )
+    {
+	*t++ = '\0'; /* Skip = char */
+	if(strcasecmp(t, "libc4") == 0)
+	{
+	    expected_type = LIB_DLL;
+	} 
+	else
+	{
+	    if(strcasecmp(t, "libc5") == 0)
+	    {
+		expected_type = LIB_ELF_LIBC5; 
+	    }
+	    else
+	    {
+		if(strcasecmp(t, "libc6") == 0)
+		{
+		    expected_type = LIB_ELF_LIBC6;
+		}
+		else
+		{
+		    warn("Unknown type field '%s' for dir '%s' - ignored\n", t, name);
+		    expected_type = LIB_ANY;
+		}
+	    }
+	}
+    }
 
     /* let 'em know what's going on */
-    if (verbose)
+    if (verbose > 0)
 	printf("%s:\n", name);
 
     /* if we can't open it, we can't do anything */
@@ -320,7 +384,7 @@ void scan_dir(char *name)
     while ((ent = readdir(dir)) != NULL)
     {
 	/* if it's not a shared library, don't bother */
-	if ((so = is_shlib(name, ent->d_name, &libtype, &islink)) == NULL)
+	if ((so = is_shlib(name, ent->d_name, &libtype, &islink, expected_type)) == NULL)
 	    continue;
 
 	/* have we already seen one with the same so name? */
@@ -426,7 +490,7 @@ int main(int argc, char **argv)
     prog = argv[0];
     opterr = 0;
 
-    while ((c = getopt(argc, argv, "DvnNXlpf:C:r:")) != EOF)
+    while ((c = getopt(argc, argv, "DvqnNXlpf:C:r:")) != EOF)
 	switch (c)
 	{
 	case 'D':
@@ -438,6 +502,10 @@ int main(int argc, char **argv)
 	case 'v':
 	    verbose = 1;	/* verbose mode */
 	    break;
+	case 'q':
+	    if (verbose <= 0)
+	        verbose = -1;	/* quiet mode */
+	    break;
 	case 'n':
 	    nodefault = 1;	/* no default dirs */
 	    nocache = 1;
@@ -482,7 +550,7 @@ int main(int argc, char **argv)
     }
 
     /* allow me to introduce myself, hi, my name is ... */
-    if (verbose)
+    if (verbose > 0)
 	printf("%s: version %s\n", argv[0], VERSION);
 
     if (printcache)
@@ -501,7 +569,7 @@ int main(int argc, char **argv)
 	    /* split into directory and file parts */
 	    if (!(cp = strrchr(argv[i], '/')))
 	    {
-		dir = "";	/* no dir, only a filename */
+		dir = ".";	/* no dir, only a filename */
 		cp = argv[i];
 	    }
 	    else
@@ -514,7 +582,7 @@ int main(int argc, char **argv)
 	    }
 
 	    /* we'd better do a little bit of checking */
-	    if ((so = is_shlib(dir, cp, &libtype, &islink)) == NULL)
+	    if ((so = is_shlib(dir, cp, &libtype, &islink, LIB_ANY)) == NULL)
 		error("%s%s%s is not a shared library", dir,
 		      (*dir && strcmp(dir, "/")) ? "/" : "", cp);
 
@@ -587,7 +655,7 @@ static int liblistcomp(liblist_t *x, liblist_t *y)
     return res;
 }
 
-void cache_dolib(char *dir, char *so, int libtype)
+void cache_dolib(const char *dir, const char *so, int libtype)
 {
     char fullpath[PATH_MAX];
     liblist_t *new_lib, *cur_lib;
@@ -701,14 +769,14 @@ void cache_print(void)
 	switch (libent[fd].flags) 
 	{
 	  case LIB_DLL:
-	    printf("(DLL)");
+	    printf("(libc4)");
 	    break;
 	  case LIB_ELF:
 	    printf("(ELF)");
 	    break;
 	  case LIB_ELF_LIBC5:
 	  case LIB_ELF_LIBC6:
-	    printf("(ELF-libc%d)", libent[fd].flags + 3);
+	    printf("(libc%d)", libent[fd].flags + 3);
 	    break;
 	  default:
 	    printf("(unknown)");
